/*
	Filter for Weasel
        -----------------

	Filter for fake user "unknown". Public alias "*" for "unknown"
        must be present.

	You must have RXU.DLL in your LIBPATH.  RXU, a Rexx
	library written by Dave Boll, is available from Hobbes
	with the name rxu1a.zip.  (But is possible that you
	already have it for use with another Rexx utility.)

						Vasilkin Andrey, 2005 y.
*/

/****************************************************************/
/*                         Settings                             */
/****************************************************************/

MyDomain = 'SAHEN.ELEKTRA.RU'      /* upper case!                           */
RealaySMTP = 'ns'                  /* relay SMTP name in our domain,
                                    default - will try read from weasel.ini */
MySMTP = 'yes1-gw'                 /* optional, default MySMTP=%HOSTNAME%   */
WeaselPath = 'c:\programs\weasel'  /* path to weasel.ini, 
					default - script source path      */

/****************************************************************/
/*                       MAIN PROGRAM                           */
/****************************************************************/

CALL RxFuncAdd SysLoadFuncs, rexxutil, sysloadfuncs
CALL SysLoadFuncs

Globals = 'ReturnPath ForwardPath InFile RealaySMTP MyDomain MySMTP WeaselPath UpCase LowCase'

/* ---------------------- Initialization ---------------------- */

if MySMTP='' then MySMTP = value('HOSTNAME',,'OS2ENVIRONMENT')

if WeaselPath='' then
do
  PARSE SOURCE rubbish rubbish WeaselPath
  j = LASTPOS('\', WeaselPath)
  if j > 0 THEN WeaselPath = LEFT(WeaselPath, j)
end
if right(WeaselPath,1)\='\' then WeaselPath = WeaselPath || '\'

Nul = D2C(0)
MailRoot = SysIni(WeaselPath'WEASEL.INI', '$SYS', 'MailRoot')
if MailRoot = 'ERROR:' then FilterError("Can't read WEASEL.INI")
j = pos(Nul,MailRoot)
if j > 0 then MailRoot = left(MailRoot, j-1)
MailRoot = STRIP(MailRoot)
MailRoot = TRANSLATE(MailRoot, '\', '/')

if RealaySMTP='' then
do
  RealaySMTP = SysIni(WeaselPath'WEASEL.INI', '$SYS', 'ForwardRelay')
  if RealaySMTP = 'ERROR:' then FilterError("Can't read ForwardRelay from WEASEL.INI")
  j = pos(Nul,RealaySMTP)
  if j > 0 then RealaySMTP = left(RealaySMTP, j-1)
  if pos('.',RealaySMTP)\=0 then
  do
    PARSE VALUE RealaySMTP WITH RealaySMTP '.' rubbish
    if translate(rubbish)\=MyDomain then RealaySMTP = ''
  end
  if RealaySMTP='' | datatype(RealaySMTP)\='CHAR' then FilterError("RealaySMTP not defined")
end



PARSE ARG InFile AddrFile

if right(AddrFile,4)='.###' then /* workaround for different versions */
do
  temp = InFile
  InFile = AddrFile
  AddrFile = temp
end

/* Copy the original input file to a new file, inserting '.' */
/* fill if needed.                                           */
/*
NewFile = SysTempFileName(MailRoot||'?????.tmp')
if NewFile = '' then FilterError("Can't create temporary file")

do forever
  buffer = LINEIN(InFile)
  if STREAM(InFile, 'S') \= 'READY' then leave
  if LEFT(buffer, 1) = '.' then call charout NewFile, '.'
  call lineout NewFile, buffer
end
call stream NewFile, 'C', 'CLOSE'
call stream InFile, 'C', 'CLOSE'
call SysFileDelete InFile
InFile = NewFile*/

/* Test with global filter (Weasel don't doing it for "unknown" yet) */
FilterProg4 = SysIni(WeaselPath'WEASEL.INI', '$SYS', 'FilterProg4')
j = pos(Nul,FilterProg4)
if j > 0 then FilterProg4 = left(FilterProg4, j-1)
if FilterProg4\='' then
do
  'call 'FilterProg4' 'InFile' 'AddrFile
  if rc\=0 then
  do
    call SysFileDelete InFile
    exit 2
  end
end

call CheckHeader

if (ForwardPath\='') then
do
  say '|Return path: 'ReturnPath
  say '|Forward as: 'ForwardPath

  TmpFile = SysTempFileName(MailRoot||'forward\?????.tmp')
  if TmpFile = '' then FilterError("Missing forward directory")

  /* Make weasel-header */
  CALL CHAROUT TmpFile, 'V000'||Nul||Nul||Nul||Nul||Nul||'1'X
  CALL CHAROUT TmpFile, ReturnPath'('ForwardPath')'
  CALL stream TmpFile, 'C', 'CLOSE'

  '@COPY 'TmpFile'/A+'InFile' 'TmpFile'/B >NUL'
  '@RENAME 'TmpFile' 'SysTempFileName('?????.FWD')

  /* Tell Weasel to re-check for outgoing mail. */
  call rxfuncadd 'rxuinit','rxu','rxuinit'
  call rxuinit
  SemName = "\SEM32\WEASEL\FORWARDMAIL"
  if RxOpenEventSem(hev, SemName) \= 0 then
    rc = RxCreateEventSem( hev ,'Shared', SemName, 'Reset')
  call RxPostEventSem hev
  call RxResetEventSem hev
  call RxCloseEventSem hev
  call RxuTerm
end

call SysFileDelete InFile
exit 2

/****************************************************************/
/*      GETTING "To:" AND/OR "From:" LINES FROM THE HEADER      */
/* Expand forward addresses in our domain for relay SMTP,       */
/* ignore for local users (test via weasel.ini).                */
/* Expand return address in our domain for local SMTP.          */
/****************************************************************/

CheckHeader: PROCEDURE EXPOSE (Globals)
  InHeader = 1
  FirstLine = 1
  ForwardPath = ''
  ReturnPath = ''
  CurField = ''

  do while InHeader
    line = LineIn(InFile)
    if (line = "") & \FirstLine then InHeader = 0
    else if (LEFT(line,1) = ' ' | LEFT(line,1) = x2c('09')) then
      do
        if CurField = 'TO' then call AddForwardPath line
        else if CurField = 'FROM' then call AddReturnPath line 
      end
    else if (POS(':', line) = 0) & \FirstLine then InHeader = 0
    else
      do
        CurField = ''
        PARSE VALUE line WITH keyword ':' remainder
        keyword = TRANSLATE(keyword)
        if (keyword="FROM") | (keyword="RETURN-PATH") then
        do
          CurField = 'FROM'
          call AddReturnPath remainder
        end
        else if (keyword = "TO") | (keyword = "CC") | (keyword = "BCC") then
        do
          CurField = 'TO'
          call AddForwardPath remainder
        end
      end
    FirstLine = 0
  end
  call stream InFile, 'C', 'CLOSE'
return

AddForwardPath: PROCEDURE EXPOSE (Globals)
  line = arg(1)
  do forever
    P = pos(',',line)
    if P=0 then item = line; else item = left(line,P-1)
    if item\='' then
    do
      item = CleanupAddress(strip(strip(item,'L',x2c('09'))))
      parse value item with UserAddr '@' UserDomain
      if UserAddr\='' & UserDomain\='' then
      do
        NewItem = 1
        if UserDomain='' then
        do
          if IsMyUser(UserAddr) then UserDomain = MyDomain
          else UserDomain = RealaySMTP||'.'||MyDomain
        end
        else if translate(UserDomain) = MyDomain then
        do
          if IsMyUser(UserAddr) then NewItem = 0
          else UserDomain = RealaySMTP||'.'||UserDomain
        end
        else NewItem = 0

        if NewItem then
        do
          item = UserAddr||'@'||UserDomain
          if ForwardPath='' then ForwardPath = item
          else ForwardPath = ForwardPath','item
        end
      end
    end
    if P=0 then leave
    line = substr(line,P+1)
  end
return

AddReturnPath: PROCEDURE EXPOSE (Globals)
  line = arg(1)
  if ReturnPath\='' then return

  ReturnPath = CleanupAddress(line)
  parse value ReturnPath with UserAddr '@' UserDomain
  if UserAddr\='' then
  do
    if UserDomain='' then
    do
      if IsMyUser(UserAddr) then ReturnPath = UserAddr||'@'||MySMTP||'.'||MyDomain
      else ReturnPath = UserAddr||'@'||MyDomain
    end
    else if translate(UserDomain)=MyDomain then
      if IsMyUser(UserAddr) then ReturnPath = UserAddr||'@'||MySMTP||'.'||UserDomain
  end
return

IsMyUser: PROCEDURE EXPOSE WeaselPath
  UpCase  = 'QWERTYUIOPASDFGHJKLZXCVBNM'
  LowCase = 'qwertyuiopasdfghjklzxcvbnm'
  rc = SysIni(WeaselPath'WEASEL.INI', translate(arg(1),LowCase,UpCase), 'Active')
return rc\='ERROR:'


/****************************************************************/
/*                 TIDYING UP AN E-MAIL ADDRESS                 */
/****************************************************************/

/*CleanupAddress: PROCEDURE
    line = STRIP(arg(1))
    if LEFT(line,1) = '"' then parse var line '"' rubbish '"' line
    if RIGHT(line,1) = ')' then parse var line line '(' rubbish ')'
    line = strip(line)
    if RIGHT(line,1) = '>' then parse value line with rubbish '<' line '>'
return line*/

CleanupAddress: PROCEDURE
  parse arg User '@' Domain
  User = Word(User,Words(User))
  Domain = Word(Domain,1)
return strip(strip(User'@'Domain,'L','<'),'T','>')


/****************************************************************/

FilterError:
  line = arg(1)
  say 'UNKNOWN-Filter: 'line
exit 1
